==> gsp-0.3.0.tgz: package/lib/pull.js <==
'use strict';

var async = require('async');
var chalk = require('chalk');
var child_process = require('child_process');
var fs = require('fs-extra');
var http = require('http');
var path = require('path');
var prompt = require('prompt');

var rsshuser = /^(?!git@)(?:ssh:\/\/)?([\w-.]+)@/;

var warns = [];
var errors = [];

var findFileSync = function findFileSync(filename, start, stop) {
    var filepath = path.join(start, filename),
        last = null;
    while (start !== stop && start !== last) {
        if (fs.existsSync(filepath)) {
            return filepath;
        }
        last = start;
        start = path.dirname(start);
        filepath = path.join(start, filename);
    }
};

var clone = function clone(repos, options, callback) {
    async.eachLimit(repos, 5, function (repo, callback) {
        repo = repo.replace(rsshuser, function (s, p) {
            return s.replace(p, options.user);
        });
        console.log('Cloning ' + repo);
        child_process.exec('git clone ' + repo, function (err, stdout, stderr) {
            if (err) {
                errors.push({
                    repo: repo,
                    msg: stderr
                });
            }
            callback();
        });
    }, callback);
};

var update = function update(repos, options, callback) {
    var rMaster = /^\*\smaster\b/m;
    async.eachLimit(repos, 5, function (repo, callback) {
        child_process.exec('git branch', {
            cwd: repo
        }, function (err, stdout) {
            console.log('Updating ' + repo);
            if (rMaster.test(stdout)) {
                child_process.exec('git pull origin master', {
                    cwd: repo
                }, function (err, stdout, stderr) {
                    if (err) {
                        errors.push({
                            repo: repo,
                            msg: stderr
                        });
                    }
                    callback();
                });
            } else {
                warns.push({
                    repo: repo,
                    msg: 'Not in master branch, update skipped.'
                });
                callback();
            }
        });
    }, callback);
};

var pull = function pull(repos, options) {
    var cloneRepos = [],
        updateRepos = [];
    async.series([function (callback) {
        repos.forEach(function (remoteUrl) {
            var repo = path.basename(remoteUrl, '.git');
            if (fs.existsSync(repo)) {
                updateRepos.push(repo);
            } else {
                cloneRepos.push(remoteUrl);
            }
        });
        callback();
    }, function (callback) {
        var hasPlaceholder = cloneRepos.some(function (u) {
            return rsshuser.test(u);
        });
        if (hasPlaceholder && !options.user) {
            prompt.message = chalk.green('[?]');
            prompt.delimiter = ' ';
            prompt.start();
            prompt.get([{
                'name': 'username',
                'message': 'Enter username to clone',
                'required': true
            }], function (err, result) {
                options.user = result.username;
                callback();
            });
        } else {
            callback();
        }
    }, function (callback) {
        clone(cloneRepos, options, callback);
    }, function (callback) {
        update(updateRepos, options, callback);
    }], function () {
        if (warns.length) {
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = warns[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var log = _step.value;

                    console.log('\n' + chalk.bold.bgYellow(' WARN ') + ' ' + log.repo);
                    console.log(log.msg);
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator['return']) {
                        _iterator['return']();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }
        }
        if (errors.length) {
            console.log(chalk.bold.red('\nPull finished with errors.'));
            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = errors[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var log = _step2.value;

                    console.log('\n' + chalk.bold.white.bgRed(' ERROR ') + ' ' + log.repo);
                    console.log(log.msg);
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2['return']) {
                        _iterator2['return']();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }
        } else {
            console.log(chalk.bold.green('\nAll repositories is up to date.'));
        }
    });
};

exports.pull = function (options) {
    var cwd = process.cwd();
    var workspace = findFileSync('.gsp_workspace', cwd);
    if (workspace) {
        process.chdir(path.dirname(workspace));
    } else {
        prompt.message = chalk.green('[?]');
        prompt.delimiter = ' ';
        prompt.start();
        prompt.get([{
            'name': 'workspace',
            'message': 'Clone all repositories to ' + cwd + '?',
            'default': 'Y/n'
        }], function (err, result) {
            if (result && result.workspace.toLowerCase() !== 'n') {
                fs.writeFileSync(path.join(cwd, '.gsp_workspace'), '');
                exports.pull(options);
            } else {
                console.log('\nPull cancelled.');
            }
        });
        return;
    }
    http.get({
        host: process.env.GSP_HOST || 'gsp.com',
        path: '/gsp/pull'
    }, function (response) {
        var data = '';
        response.on('data', function (chunk) {
            data += chunk;
        });
        response.on('end', function () {
            try {
                data = JSON.parse(data);
            } catch (e) {
                console.log(chalk.red('The response is not a valid JSON string of repositories.'));
                process.exit();
            }
            pull(data, options);
        });
    });
};

==> gsp-0.4.2.tgz: package/lib/pull.js <==
'use strict';

var async = require('async');
var chalk = require('chalk');
var child_process = require('child_process');
var fs = require('fs-extra');
var http = require('http');
var path = require('path');
var prompt = require('prompt');

var warns = [];
var errors = [];

var findFileSync = function findFileSync(filename, start, stop) {
    var filepath = path.join(start, filename),
        last = null;
    while (start !== stop && start !== last) {
        if (fs.existsSync(filepath)) {
            return filepath;
        }
        last = start;
        start = path.dirname(start);
        filepath = path.join(start, filename);
    }
};

var clone = function clone(repos, options, callback) {
    var rUsername = /x{5}/;
    async.eachLimit(repos, 5, function (repo, callback) {
        repo = repo.replace(rUsername, options.user);
        console.log('Cloning ' + repo);
        child_process.exec('git clone ' + repo, function (err, stdout, stderr) {
            if (err) {
                errors.push({
                    repo: repo,
                    msg: stderr
                });
            }
            callback();
        });
    }, callback);
};

var update = function update(repos, options, callback) {
    var rMaster = /^\*\smaster\b/m;
    async.eachLimit(repos, 5, function (repo, callback) {
        child_process.exec('git branch', {
            cwd: repo
        }, function (err, stdout) {
            console.log('Updating ' + repo);
            if (rMaster.test(stdout)) {
                child_process.exec('git pull origin master', {
                    cwd: repo
                }, function (err, stdout, stderr) {
                    if (err) {
                        errors.push({
                            repo: repo,
                            msg: stderr
                        });
                    }
                    callback();
                });
            } else {
                warns.push({
                    repo: repo,
                    msg: 'Not in master branch, update skipped.'
                });
                callback();
            }
        });
    }, callback);
};

var pull = function pull(repos, options) {
    var cloneRepos = [],
        updateRepos = [];
    async.series([function (callback) {
        repos.forEach(function (remoteUrl) {
            var repo = path.basename(remoteUrl, '.git');
            if (fs.existsSync(repo)) {
                updateRepos.push(repo);
            } else {
                cloneRepos.push(remoteUrl);
            }
        });
        callback();
    }, function (callback) {
        var hasPlaceholder = cloneRepos.some(function (u) {
            return u.indexOf('ssh://xxxxx@') === 0;
        });
        if (hasPlaceholder && !options.user) {
            prompt.message = chalk.green('[?]');
            prompt.delimiter = ' ';
            prompt.start();
            prompt.get([{
                'name': 'username',
                'message': 'Enter username to clone',
                'required': true
            }], function (err, result) {
                options.user = result.username;
                callback();
            });
        } else {
            callback();
        }
    }, function (callback) {
        clone(cloneRepos, options, callback);
    }, function (callback) {
        update(updateRepos, options, callback);
    }], function () {
        if (warns.length) {
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = warns[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var log = _step.value;

                    console.log('\n' + chalk.bold.bgYellow(' WARN ') + ' ' + log.repo);
                    console.log(log.msg);
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator['return']) {
                        _iterator['return']();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }
        }
        if (errors.length) {
            console.log(chalk.bold.red('\nPull finished with errors.'));
            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = errors[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var log = _step2.value;

                    console.log('\n' + chalk.bold.white.bgRed(' ERROR ') + ' ' + log.repo);
                    console.log(log.msg);
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2['return']) {
                        _iterator2['return']();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }
        } else {
            console.log(chalk.bold.green('\nAll repositories is up to date.'));
        }
    });
};

exports.pull = function (options) {
    var cwd = process.cwd();
    var workspace = findFileSync('.gsp_workspace', cwd);
    if (workspace) {
        process.chdir(path.dirname(workspace));
    } else {
        prompt.message = chalk.green('[?]');
        prompt.delimiter = ' ';
        prompt.start();
        prompt.get([{
            'name': 'workspace',
            'message': 'Clone all repositories to ' + cwd + '?',
            'default': 'Y/n'
        }], function (err, result) {
            if (result && result.workspace.toLowerCase() !== 'n') {
                fs.writeFileSync(path.join(cwd, '.gsp_workspace'), '');
                exports.pull(options);
            } else {
                console.log('\nPull cancelled.');
            }
        });
        return;
    }
    http.get({
        host: process.env.GSP_HOST || 'gsp.com',
        path: '/gsp/pull'
    }, function (response) {
        var data = '';
        response.on('data', function (chunk) {
            data += chunk;
        });
        response.on('end', function () {
            try {
                data = JSON.parse(data);
            } catch (e) {
                console.log(chalk.red('The response is not a valid JSON string of repositories.'));
                process.exit();
            }
            pull(data, options);
        });
    });
};
